package org.freecompany.imp;

import org.freecompany.brimstone.module.Module;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;
import org.apache.tools.ant.BuildException;

import static java.util.logging.Level.FINE;
import static java.util.logging.Level.WARNING;
import static org.freecompany.imp.Constants.TYPE_COMPILE;
import static org.freecompany.imp.Constants.TYPE_COMPONENT;
import static org.freecompany.imp.Constants.TYPE_RUN;
import static org.freecompany.imp.Constants.TYPE_TEST;
import static org.freecompany.imp.Util.getJavadocs;
import static org.freecompany.imp.Util.getModules;

public class Dependency extends Resolve {

	protected final Logger logger = Logger.getLogger( Dependency.class.getName());
	protected boolean isModule;
	protected boolean imported;
	protected List< String> types;

	private boolean canImport( final String file) {
		return file.endsWith( "jar") || file.endsWith( "module");
	}

	/**
	 * Resolves the nested (and index included) dependencies into the proper runtime path sets.  This has
	 * to be done as a separate step because the types values will not be set at the time the objects are
	 * added to this container.
	 */
	public void execute() throws BuildException {
		setupAll();
		super.execute();

		try {
			final Object object = Module.load( url);
			if ( object instanceof Module) {
				logger.log( FINE, "Importing module definition '" + url + "'.");
				isModule = true;
				final Module module = ( Module) object;
				getModules( getProject()).add( module);

				final String javadoc = module.getJavadoc();
				if ( javadoc != null) {
					logger.log( FINE, "Including Javadoc URL {0}.", javadoc);
					getJavadocs( getProject()).add( javadoc);
				}
				for ( URL url : module.getComponents()) if ( canImport( url.getFile())) addDependency( url);
				for ( URL url : module.getDependencies()) if ( canImport( url.getFile())) addDependency( url);
			} else {
				if ( object instanceof URL) {
					logger.log( FINE, "Loading module fragment '" + object + "'.");
					uri = object.toString();
					super.execute();
				} else logger.log( FINE, "Including plain url {0}.", url);

				if ( types.contains( TYPE_COMPILE)) Util.getCompile( getProject()).add( this);
				if ( types.contains( TYPE_TEST)) Util.getTest( getProject()).add( this);
				if ( types.contains( TYPE_RUN)) Util.getRun( getProject()).add( this);
			}

			if ( types.contains( TYPE_RUN)) if ( !imported) Util.getPackage( getProject()).add( this);
			if ( types.contains( TYPE_COMPONENT)) Util.getComponents( getProject()).add( uri);
		} catch ( MalformedURLException e) {
			throw new BuildException( e);
		} catch ( IOException e) {
			throw new BuildException( e);
		}
	}

	protected void addDependency( final URL dependency) {
		logger.log( FINE, "Importing module dependency ''{0}''.", dependency);
		Dependency task = new Dependency();
		task.setName( taskName);
		task.setProject( getProject());
		task.setImported( true);
		task.setUri( dependency.toString());
		
		final ArrayList< String> copy = new ArrayList< String>( types);
		copy.remove( TYPE_COMPONENT);
		
		task.setTypes( copy);
		task.execute();
	}

	public void setupAll() {
		Util.getAll( getProject()).add( this);
	}

	public boolean isModule() { return isModule; }

	/**
	 * Sets this dependency to imported mode, which causes the dependency to be
	 * excluded from the final list of packaged dependencies.
	 */
	public void setImported( boolean imported) {
		this.imported = imported;
	}

	public void setTypes( final String types) {
		setTypes( Arrays.asList( types.split( ",")));
	}

	public void setTypes( final List< String> types) {
		this.types = types;
	}
	
	public List< String> getTypes() throws BuildException {
		if ( types == null) throw new BuildException( "Types is a required attribute.");
		return types;
	}

	public String toString() {
		return "Dependency[ " + url + "]";
	}

	protected String getBinding() {
		return name;
	}
}
