package org.freecompany.imp;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;
import org.apache.tools.ant.*;
import org.apache.tools.ant.taskdefs.*;

import static java.util.logging.Level.*;
import static org.freecompany.imp.Constants.REFERENCE_IMPORTS;

/**
 * Extension of the built in Ant import task that allows a imported Ant build file to be
 * resovled and downloaded from configured repositories.  This task actively tries to figure
 * out if it's at the top level and fail if so, so we cannot simply delegate to a new instance.
 */
public class Import extends ImportTask {

	protected Logger logger = Logger.getLogger( Import.class.getName());
	protected String uri;

	@SuppressWarnings( "unchecked")
	public void execute() throws BuildException {
		final Project project = getProject();
		Set< String> completed;
		synchronized ( project) {
	   		completed = ( Set< String>) project.getReference( REFERENCE_IMPORTS);
			if ( completed == null) {
				completed = new HashSet< String>();
				project.addReference( REFERENCE_IMPORTS, completed);
			}
		}

		if ( completed.contains( uri)) {
			logger.log( FINE, "Ignoring duplicate import request for ''{0}''.", uri);
			return;
		}
		completed.add( uri);

		logger.log( FINE, "Importing URL ''{0}'' within classloader ''{1}''.", new Object[] { uri, getClass().getClassLoader()});
		Resolve task = new Resolve() { public String getBinding() { return name; }};
		task.setProject( getProject());
		task.setUri( uri);
		task.execute();
		logger.log( FINE, "Imported fragment from ''{0}'' at ''{1}''.", new Object[] { task.getUrl(), task.getPath()});
		setFile( task.getPath());
		super.execute();
	}

	public void setUri( String uri) {
		this.uri = uri;
	}
}
