package org.freecompany.imp;

import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.net.MalformedURLException;
import java.net.URL;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Collections;
import java.util.Properties;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.BuildException;
import org.freecompany.brimstone.core.URNConnection;
import org.freecompany.brimstone.main.DefaultListener;
import org.freecompany.brimstone.module.Module;
import org.xml.sax.SAXException;

import static java.util.Collections.list;
import static org.apache.tools.ant.Project.MSG_DEBUG;
import static org.freecompany.imp.Constants.*;

public class Util {

	private final static ThreadLocal< Project> local = new ThreadLocal< Project>();

	public static void setProject( final Project project) { local.set( project); }
	public static Project getProject() { return local.get(); }

	/**
	 * Loads a URL while dereferencing any module fragments.
	 */
	public static URL loadUrl( final Project project, final String uri) throws IOException, MalformedURLException {
		project.log( "Loading URL '" + uri + "'.", MSG_DEBUG);
		final URL url = new URL((( Module) project.getReference( REFERENCE_MODULE)).getBase(), uri);
		final Object object = Module.load( url);
		if ( object instanceof URL) return ( URL) object;
		return url;
	}

	public static void initialize( final Project project) throws BuildException {
		synchronized ( project) {
			if ( Boolean.parseBoolean( project.getProperty( PROPERTY_INITIALIZED))) return;
			project.setProperty( PROPERTY_INITIALIZED, Boolean.TRUE.toString());
		}

		if ( URNConnection.getListener() == null) URNConnection.addListener( new DefaultListener());

		setProject( project);
		try {
			loadProperties( project);
			final String module = project.getProperty( PROPERTY_RUNTIME);
			if ( module == null) throw new BuildException( "Cound not load default properties.");
			project.addReference( REFERENCE_MODULE, new Module( new URL( module), true));
		} catch ( SAXException e) {
			throw new BuildException( e);
		} catch ( IOException e) {
			throw new BuildException( e);
		}

		project.addReference( REPLACE_PROPERTIES, new Properties());
	}

	public static void prepareReplacements( final Project project) throws IOException {
		final File replacing = File.createTempFile( "replacing", "properties");
		replacing.deleteOnExit();
		
		final Properties properties = ( Properties) project.getReference( REPLACE_PROPERTIES);
		final OutputStream output = new FileOutputStream( replacing);
		properties.store( output, "Generated by Imp.  This file can be deleted once the build completes.");
		output.close();

		project.setProperty( REPLACE_PROPERTY, replacing.getAbsolutePath());
	}

	/**
	 * Sets a property both in the project and in the list of replace filters to be used
	 * during file copies in the build.
	 */
	public static void setProperty( final Project project, final String key, final String value) {
		project.setProperty( key, value);
		(( Properties) project.getReference( REPLACE_PROPERTIES)).setProperty( key, value);
	}

	private static void loadProperties( final Project project) throws IOException {
		loadProperties( project, System.getProperty( "user.home") + File.separator + ".imp" + File.separator + "imp.properties", false);
		loadProperties( project, "build.properties", true);
		loadProperties( project, Util.class.getClassLoader().getResource( "org/freecompany/imp/default.properties"), false);
	}

	public static void loadProperties( final Project project, final String path, final boolean overwrite) throws IOException {
		project.log( "Attempting to load '" + path + "' with overwrite '" + overwrite + "'.", MSG_DEBUG);
		final File file = new File( path);
		if ( !file.exists()) return;
		loadProperties( project, file.toURL(), overwrite);
	}

	@SuppressWarnings({ "unchecked"})
	public static void loadProperties( final Project project, final URL url, final boolean overwrite) throws IOException {
		final Properties properties = new Properties();
		properties.load( url.openStream());
		final Enumeration< String> enumeration = ( Enumeration< String>) properties.propertyNames();
		for ( Iterator< String> i = list( enumeration).iterator(); i.hasNext();) {
			final String name = i.next();
			project.log( "Adding new property'" + name + "'.", MSG_DEBUG);
			if ( overwrite) project.setUserProperty( name, properties.getProperty( name));
			else project.setNewProperty( name, properties.getProperty( name));
		}
	}
	
	@SuppressWarnings({ "unchecked", "deprecated"})
	public static < T> Collection< T> get( Project project, String key) {
		Collection list = ( Collection) project.getReference( key);
		if ( list == null) {
			list = new LinkedHashSet();
			project.addReference( key, list);
		}
		return list;
	}

	public static Collection< Dependency> getAll( Project project) {
		return get( project, REFERENCE_ALL);
	}

	public static Collection< Dependency> getCompile( Project project) {
		return get( project, REFERENCE_COMPILE);
	}

	public static Collection< Dependency> getTest( Project project) {
		return get( project, REFERENCE_TEST);
	}

	public static Collection< Dependency> getRun( Project project) {
		return get( project, REFERENCE_RUN);
	}

	public static Collection< Dependency> getPackage( Project project) {
		return get( project, REFERENCE_PACKAGE);
	}

	public static Collection< String> getComponents( Project project) {
		return get( project, REFERENCE_COMPONENTS);
	}

	public static Collection< String> getTestComponents( Project project) {
		return get( project, REFERENCE_TEST_COMPONENTS);
	}

	public static Collection< Module> getModules( Project project) {
		return get( project, REFERENCE_MODULES);
	}

	public static Collection< PropertyTask> getProperties( Project project) {
		return get( project, REFERENCE_PROPERTIES);
	}

	public static Collection< String> getJavadocs( Project project) {
		return get( project, REFERENCE_JAVADOCS);
	}
}
