package org.freecompany.imp.index;

import org.freecompany.imp.*;
import org.apache.tools.ant.*;
import org.apache.tools.ant.taskdefs.*;
import org.w3c.dom.*;
import java.io.*;
import java.util.logging.*;
import javax.xml.xpath.*;

import static javax.xml.xpath.XPathConstants.*;

public class Magic extends Generator {

	protected XPath path = XPathFactory.newInstance().newXPath();

	/**
	 * Builds the current element.  If this element is found to correspond to the key
	 * of the root element, then we act to build all subelements.
	 */
	protected void generate( final Document document) throws Exception {
		final Element root = document.getDocumentElement();
		final boolean base = this.name.equals( path.evaluate( "@key", root));
		
		NodeList elements = ( NodeList) path.evaluate( "project", root, NODESET);
		for ( int x = 0; x < elements.getLength(); x++) {
			Element element = ( Element) elements.item( x);
			String basedir = path.evaluate( "@basedir", element);
			String value = path.evaluate( "info/name/text()", element);
			
			if ( this.name.equals( value)) buildIndex( element);
			else if ( base) buildProject( basedir);
			else System.out.println( "Skipping non matching subelement '" + value + "'.");
		}
	}

	protected void buildProject( String directory) throws Exception {
		System.out.println( "Registering element build for '" + directory + "'.");
		AntOverride ant = new AntOverride();
		ant.setDir( new File( directory));
		index.addAnt( ant);
	}

	protected void buildIndex( Element element) throws Exception {
		index.setGroup( path.evaluate( "info/group/text()", element));
		buildPackagingType( element);
		NodeList includes = ( NodeList) path.evaluate( "dependencies/include", element, NODESET);
		for ( int x = 0; x < includes.getLength(); x++) {
			Element include = ( Element) includes.item( x);
			String key = path.evaluate( "@key", include);
			System.out.println( "Loading dependency '" + key + "'.");
			boolean build = include.hasAttribute( "build") ? Boolean.valueOf( path.evaluate( "@build", include)).booleanValue() : true;
			boolean test = include.hasAttribute( "test") ? Boolean.valueOf( path.evaluate( "@test", include)).booleanValue() : true;
			boolean run = include.hasAttribute( "run") ? Boolean.valueOf( path.evaluate( "@run", include)).booleanValue() : true;
			StringBuffer buffer = new StringBuffer();
			buildTypes( buffer, build, test, run);
			buildDependency( key, buffer.toString(), element);
		}
	}

	protected void buildPackagingType( Element element) throws Exception {
		StringBuffer buffer = new StringBuffer();
		addType( path.evaluate( "info/type/text()", element), buffer);
		
		NodeList types = ( NodeList) path.evaluate( "info/types/type", element, NODESET);
		if ( types.getLength() > 0) for ( int x = 0; x < types.getLength(); x++) addType( path.evaluate( "text()", types.item( x)), buffer);
		
		if ( buffer.length() == 0) buffer.append( "jar");
		index.setTypes( buffer.toString());
	}

	protected void addType( String type, StringBuffer buffer) {
		if ( type == null) return;
		if ( buffer.length() > 0) buffer.append( ",");
		buffer.append( type);
	}

	protected void buildDependency( String key, String types, Node element) throws Exception {
		String resource = "../resource[ info/name/text() = '" + key + "']";
		String uri = "urn:brimstone:";
		uri += path.evaluate( resource + "/info/group/text()", element);
		uri += "/" + path.evaluate( resource + "/info/name/text()", element);
		uri += "-" + path.evaluate( resource + "/info/version/text()", element);
		uri += ".jar";
		Dependency dependency = new Dependency();
		dependency.setProject( project);
		dependency.setTypes( types);
		dependency.setUri( uri);
		index.addDependency( dependency);
	}

	protected void buildTypes( StringBuffer buffer, boolean compile, boolean test, boolean run) {
		if ( compile) buildBuffer( buffer, "compile");
		if ( test) buildBuffer( buffer, "test");
		if ( run) buildBuffer( buffer, "run");
	}

	protected void buildBuffer( StringBuffer buffer, String value) {
		if ( buffer.length() != 0) buffer.append( ",");
		buffer.append( value);
	}
}
